// This program allow to test RAMPS boards
// Ce programme permet de tester les cartes RAMPS
//
// You need to perform all motor wiring before testing it, including driver motors.
// Please do not plug thermistor, endstops, heated bed and heating cartdridge
//
// Vous aurez besoin de câbler les moteurs et les stepsticks (branchez les hors tension, votre carte doit être sans alimentation pour ce faire) 
// Ne branchez pas les thermistances ni les endstops et rebranchez ensuite votre alimentation pour faire les tests
// 
// Programme développé par Hugo Flye @ eMotion Tech
// Documentation complétée par Anthony BERNA @ eMotion Tech
// Licence CC BY-NC-SA 2.0 FR

// ---------- PROCESS ----------
//
// 1°) unplug endstops, thermistors and heating elements, only motors and stepsticks must remain on the RAMPS board
// 1°) débrancher endstops, thermistances et éléments de chauffe, il ne doit rester que moteurs et pilotes moteurs sur la carte RAMPS
//
// 2°) with Arduino software, upload this program in the Arduino MEGA 2560 board
// 2°) téléverser ce programme dans la carte Arduino MEGA 2560 à l'aide du logiciel Arduino
//
// 3°) plug power supply to the RAMPS card
// 3°) brancher l'alimentation sur la carte RAMPS
//
// 4°) at the beginning of the program, LED 8, 9 and 10 must flash : indicate that D8, D9 et D10 are operational
// 4°) au démarrage du programme, les LED 8, 9 et 10 doivent clignoter : indique que les borniers D8, D9 et D10 sont opérationnels
//
// 5°) motors must rotate in one direction and the other regularly : indicate that motors and stepsticks are operational
// 5°) les moteurs doivent tourner dans un sens et dans l'autre régulièrement : indique que les ports de motorisation sont opérationnels
//
// 6°) endstop test : connect pin "S" to pin "-" will make the LED 1 flashing : indicate that the endstop function is operational
// 6°) test des ports d'endstops : relier la broche "S" à la broche "-" fera clignoter la LED 1 : indique que le port testé est opérationnel
//
// 7°) thermistor test : connect each pins together will make the LED 10 flashing : indicate that the thermistor function is operational
// 7°) test des ports de thermistances : relier les 2 broches d'un port fera clignoter la LED 10 : indique que le port testé est opérationnel
//
// You can connect the USB port to check in the Arduino serial monitor (magnifying glass, top right) which can indicate which elements are short-circuited
// Vous pouvez brancher le port USB pour verifier dans Arduino le Moniteur serie (loupe en haut à droite) qui peut indiquer quels éléments sont en court-circuit


//Check board
#ifndef __AVR_ATmega1280__
 #ifndef __AVR_ATmega2560__
 #error Oops!  Make sure you have 'Arduino Mega' selected from the 'Tools -> Boards' menu.
 #error Oops!  Vous avez un souci de selection de carte 'Outils-> type de carte -> arduino mega 2560'
 #endif
#endif

// Uniquement pour l'Arduino Mega 2560
// PINOUT 2560
#define X_STEP_PIN         54
#define X_DIR_PIN          55
#define X_ENABLE_PIN       38
#define X_MIN_PIN          3
#define X_MAX_PIN          2

#define Y_STEP_PIN         60
#define Y_DIR_PIN          61
#define Y_ENABLE_PIN       56
#define Y_MIN_PIN          14
#define Y_MAX_PIN          15

#define Z_STEP_PIN         46
#define Z_DIR_PIN          48
#define Z_ENABLE_PIN       62
#define Z_MIN_PIN          18
#define Z_MAX_PIN          19

#define E0_STEP_PIN         26
#define E0_DIR_PIN          28
#define E0_ENABLE_PIN       24

#define E1_STEP_PIN         36
#define E1_DIR_PIN          34
#define E1_ENABLE_PIN       30

#define SDPOWER            -1
#define SDSS               53
#define SDCARDDETECT 	    49

#define LED_PIN            13
#define FAN_PIN            9
#define PS_ON_PIN          12
#define KILL_PIN           -1

#define HEATER_0_PIN       10
#define HEATER_1_PIN       8
#define HEATER_2_PIN       9
#define TEMP_0_PIN         13   // ANALOG NUMBERING
#define TEMP_1_PIN         14   // ANALOG NUMBERING
#define TEMP_2_PIN         15

boolean bool_invert; //general var


// REGROUPING PINOUT
int en_all[5] = { 
  X_ENABLE_PIN,Y_ENABLE_PIN,Z_ENABLE_PIN,E0_ENABLE_PIN,E1_ENABLE_PIN};
int dir_all[5] = { 
  X_DIR_PIN,Y_DIR_PIN,Z_DIR_PIN,E0_DIR_PIN,E1_DIR_PIN};
int step_all[5] = { 
  X_STEP_PIN,Y_STEP_PIN,Z_STEP_PIN,E0_STEP_PIN,E1_STEP_PIN};
int heater_all[4] = {
  FAN_PIN ,HEATER_0_PIN,HEATER_1_PIN,HEATER_2_PIN};
int endstop_all[6] = {
  X_MIN_PIN ,X_MAX_PIN,Y_MIN_PIN ,Y_MAX_PIN,Z_MIN_PIN ,Z_MAX_PIN};
int temp_all[3] = {
  TEMP_0_PIN  ,TEMP_1_PIN  ,TEMP_2_PIN  };

String EndstopName [6] = {"Xmin","Xmax","Ymin","Ymax","Zmin","Zmax"};

// ROTATION OF STEPPER MOTOR IN STEP PER TEST
#define rot 8*200


//BLINKING PROCEDURE
//  nb = number of time blinking
//  led = led PIN
//  (temp = delay, default 1 sec)
void blink(int nb,int led,int temp = 1000) {

  for(int i=0 ; i<nb;i++) {
    digitalWrite(led, HIGH);   // turn the LED on (HIGH is the voltage level)
    delay(temp);               // wait for a second

    digitalWrite(led , LOW);    // turn the LED off by making the voltage LOW
    delay(temp);    
  }
}


//STEPPER ROTATION
//
//  step_pin = Driver motor STEP pin
//  dir_pin  = Driver motor DIRECTION pin
//  en_pin   = Driver motor ENABLE pin
//  dir      = which direction you go
//  steps    = number of step to perform
//  (temp    = delay between steps : default = 50ms)
void rotate(int step_pin, int dir_pin,int en_pin, boolean dir, int steps, int temp = 50) {

  digitalWrite(en_pin , LOW); //default ENABLE LOW because stepper driver use "not enable" input
  digitalWrite(dir_pin,dir); //set dir

    for(int i=0; i <steps;i++) { //iterate along steps and send pulses
    delayMicroseconds(temp); 
    digitalWrite(step_pin, LOW);
    delayMicroseconds(temp); 
    digitalWrite(step_pin, HIGH);
  }

  digitalWrite(en_pin , HIGH); //disable motor 
}

//ALL STEPPER ROTATION 
// in the same time, good for debug
//  dir      = which direction you go
//  steps    = number of step to perform
//  (temp    = delay between steps : default = 10ms)
void rotate_all(boolean dir,int steps,int temp = 10) {

  //INIT ALL
  for(int i = 0; i< 5 ; i++){
    digitalWrite(en_all[i] , LOW); //default ENABLE LOW because stepper driver use "not enable" input
    digitalWrite(dir_all[i],dir);  //set dir
  }
  //ROTATE ALL
  for(int a=0; a <steps;a++) {
    for(int i=0; i <5;i++) {
      delayMicroseconds(temp); 
      digitalWrite(step_all[i], LOW);
      delayMicroseconds(temp); 
      digitalWrite(step_all[i], HIGH);
    }

  }

}


//GENERAL SETUP
void setup() {

  //Init serial 
  Serial.begin(9600); 
  
  //SET INPUT
  for(int i=0;i<6;i++) {
    pinMode(endstop_all[i], INPUT);
    digitalWrite(endstop_all[i], HIGH);
  }
  Serial.println("--Test Ramps, eMotion Tech--"); 

  //SET OUTPUT
  pinMode(FAN_PIN , OUTPUT);
  pinMode(HEATER_0_PIN , OUTPUT);
  pinMode(HEATER_1_PIN , OUTPUT);
  pinMode(LED_PIN , OUTPUT);

  pinMode(X_STEP_PIN , OUTPUT);
  pinMode(X_DIR_PIN , OUTPUT);
  pinMode(X_ENABLE_PIN , OUTPUT);

  pinMode(Y_STEP_PIN , OUTPUT);
  pinMode(Y_DIR_PIN , OUTPUT);
  pinMode(Y_ENABLE_PIN , OUTPUT);

  pinMode(Z_STEP_PIN , OUTPUT);
  pinMode(Z_DIR_PIN , OUTPUT);
  pinMode(Z_ENABLE_PIN , OUTPUT);

  pinMode(E0_STEP_PIN , OUTPUT);
  pinMode(E0_DIR_PIN , OUTPUT);
  pinMode(E0_ENABLE_PIN , OUTPUT);

  pinMode(E1_STEP_PIN , OUTPUT);
  pinMode(E1_DIR_PIN , OUTPUT);
  pinMode(E1_ENABLE_PIN , OUTPUT);


  // FIRST CHECK : TEMPERATURE CHECKING, 1023 default value, should not blink if OK
  Serial.println("Test thermistance a vide"); 
  for(int k = 0 ; k < 3 ; k ++) {
    if(analogRead(temp_all[k] )<1000){
      Serial.println("Fail thermistor "+String(k)); 
      blink(10,10,100);
      blink(10,9,100);
      
    }
  }
  // Blink OK
  blink(2,LED_PIN,100);

  bool_invert = 0;
  Serial.println("Demarrage boucle"); 
}





// MAIN : GENERAL LOOP
// Teste en boucle les differents elements
void loop() {

  //Check all HEATERS, all ON
  //Alume les Mosfets (hotends, Fan , bed)
  for(int j = 0; j<4;j++) {
    digitalWrite(heater_all[j], bool_invert);
  }

  //Rotation all steppers: one direction
  //Fait tourner les moteurs dans une direction
  Serial.println("Rotation moteur"); 
  rotate_all(bool_invert ,rot);

  //check ENDSTOP
  for(int j = 0; j<6;j++) {
    while(digitalRead(endstop_all[j])==false) {
      Serial.println("OK Endstop "+EndstopName[j]); 
      blink(1,LED_PIN,100);
      
    }
  }
    //Check Thermistors
    for(int k = 0 ; k < 3 ; k ++) {
      if(analogRead(temp_all[k] )<10)
        while(analogRead(temp_all[k] )<10) {
           Serial.println("OK thermistor "+String(k)); 
          blink(1,10,100);
         
        }
    }  
  

  bool_invert = !bool_invert ; // Invert motor dir, inverse le sens des moteurs
}